;*****************************************************************************
;* Modified "Altair 680B PROM Monitor, ACIA Version 1.0"
;*      for the Mini680 Memory Expansion Card
;*****************************************************************************
	cpu	6800		; Assemble for the 6800
	relaxed on		; Allow non-intel constant formats

MIVEC		EQU	$100
NMIVEC	EQU	$104
STRAPS	EQU	$F002
NOTERM	EQU	0
ACIACS	EQU	$F000
ACIADA	EQU	$F001
;*************************
;* Monitor Stack and Flags
;*************************
	ORG	$F1
STACK		RMB 1		;BOTTOM OF MONITOR'S STACK
BRKADR	RMB 1		;BREAKPOINT ADDRESS FLAG
ECHO		RMB 1		;TTY ECHO FLAG
EXTFLG	RMB 1		;ENTENDED CHARACTER FLAG
BUFULL	RMB 1		;BUFFER FULL FLAG
SAVSTK	RMB 2		;TEMP FOR STACK POINTER
TEMP		RMB 1		;TEMPORARY STORAGE
BYTECT	RMB 1		;BYTE COUNT
XHI		RMB 1		;XREG HIGH
XLOW		RMB 1		;XREG LOW
SHIFT		RMB 1		;BAUDOT SHIFT FLAG
SAVEX		RMB 2		;TEMP FOR INDEX RG
BUFFER	RMB 1		;BAUDOT CHARACTER FLAG
	ORG	$7FF0
XTMP		RMB 2
FREAD		RMB 2		;Flash Read Location
PAGE		RMB 1		;Current Page for Flash Reads

;**************************
;* Start of Monitor Program
;**************************
	ORG	$7000		; Start of program
	LDX	#StartMsg	; Print the startup message
	JSR	OUTST		; String output subroutine
	JMP	CRLF		; Main Monitor Routine

;***************************************************
;* Input one char into register 'A'
;* Echo character if bit 7 of the ECHO flag is clear
;***************************************************
INCH	BSR	POLCAT	;ACIA STATUC TO A REG
	BCC	INCH		;RECEIVE NOT READY
	LDA	B, #$7F	;MASK FOR PARITY REMOVAL
	CMP	B, ECHO	;CHECK ECHO FLAG
	AND	B, ACIADA	;GET CARACTER
	BCC	OUTCH		;ECHO
	RTS			;NO ECHO
;********************************************
;* THE FOLLOWING NOP LINES LINES UP THE ENTRY
;* POINTS TO POLCAT IN THE TWO VERSIONS
;* OF THE MONITOR
;********************************************
	NOP
;********************************
;* INPUT ONE HEX DIGIT INTO B REG
;* RETURN TO CALLING PROGRAM IF
;* CHARACTER RECEIVED IS A HEX
;* DIGIT. IF NOT HEX, GO TO CRLF
;********************************
INHEX BSR	INCH		;GET A CHARACTER
	SUB	B, #'0'
	BMI	C1		;NOT HEX
	CMP	B, #$09
	BLE	IN1HG		;NOT HEX
	CMP	B, #$11
	BMI	C1		;NOT HEX 
	CMP	B, #$16
	BGT	C1		;NOT HEX
	SUB	B, #$07	;IT'S A LETTER-GET BCD
IN1HG	RTS			;RETURN
;**************************************
;* POLE FOR CHARACTER
;* SETS CARRY IF CHARACTER IS IN BUFFER
;* CLOBBERS 8 REG
;**************************************
POLCAT LDA	B, ACIACS	;ACIA STATUS T0 B
	ASRB			;ROTATE RDRF BIT INTO CARRY
	RTS			;RETURN
;***********************************
;* LOAD PAPER TAPE
;* LOAD ONLY S1 TYPE RECORDS 
;* TERMINATE ON S9 OR CHECKSUM ERROR
;***********************************
LOAD	BSR	INCH		;READ FRAME
	SUB	B, #'S'
	BNE	LOAD		;FIRST CHAR NOT (S)
	BSR	INCH		;READ FRAME
	CMP	B, #'9'
	BEQ	C1		;S9 END OF FILE
	CMP	B, #'1'
	BNE	LOAD		;SECOND CHAR NOT (1)
	CLR	A		;ZERO THE CHECKSUM
	BSR	BYTE		;READ BYTE
	SUB	B, #$02
	STA	B, BYTECT	;BYTE COUNT
	BSR	BADDR		;GET ADDRESS OF BLOCK
LOAD11 BSR	BYTE		;GET DATA BYTE
	DEC	BYTECT	;DECREMENT BYTE COUNT
	BEQ	LOAD15	;DONE WITH THIS BLOCK
	STA	B, 0, X	;STORE DATA
	INX			;BUMP POINTER
	BRA	LOAD11	;GO BACK FOR MORE
LOAD15 INC	A		;INCREMENT CHECKSUM
LLOAD	BEQ	LOAD		;ALL OK - IT'S ZERO
C1	BRA	CRLF		;CHECKSUM ERROR - QUIT
;***********************************
;* READ BYTE (2 HEX DIGITS)
;* INTO B REG
;* A IS USED FOR PAPER TAPE CHECKSUM
;***********************************
BYTE	BSR	INHEX		;GET FIRST HEX DIG
	ASL	B		;GET SHIFT TO HIGH ORDER 4 BITS
	ASL	B
	ASL	B
	ASL	B
	ABA			;ADD TO CHEKSUM
	STA	B, TEMP	;STORE DIGIT
	BSR	INHEX		;GET 2ND HEX DIG
	ABA			;ADD TO CHECKSUM
	ADD	B, TEMP	;COMBINE DIGITS TO GET BYTE
	RTS			;RETURN
;*********************************
;* READ 16 BIT ADDRESS INTO X
;* STORE SAME ADDRESS IN XHI & XLO
;* CLOBBERS B REG
;*********************************
BADDR BSR	BYTE		;GET HIGH ORDER ADDRESS
	STA	B, XHI	;STORE IT
	BSR	BYTE		;GET LOW ORDER ADDRESS
	STA	B, XLOW	;STORE IT
	LDX	XHI		;LOAD X WITH ADDRESS BUILT
	RTS			;RETURN
;*********************
;* PRINT BYTE IN A REG
;* CLOBBERS B REG
;*********************
OUT2H	TAB			;COPY BYTE TO B
	LSR	B		;SHIFT TO RIGHT
	LSR	B
	LSR	B
	LSR	B
	BSR	OUTHR		;OUTPUT FIRST DIGIT
	TAB			;BYTE INTO B AGAIN
OUTHR AND	B, #$0F	;GET RID OF LEFT DIG
	ADD	B, #$30	;GET ASCII
	CMP	B, #$39
	BLS	OUTCH
	ADD	B, #$07	;IF IT'S A LETTER ADD 7
	NOP			;LINE UP OUTCH ENTRY POINTS
	NOP
OUTCH	FCB	$8C		;USE CPX SKIP TRICK
OUTS	LDA	B, #$20	;OUTS PRINTS A SPACE
;******************************
;* OUTCH OUTPUTS CHARACTER IN B
;******************************
	PSH	B		;SAVE CHAR
OUTC1 BSR	POLCAT	;ACIA STATUS TO B REG
	ASR	B
	BCC	OUTC1		;XMIT NOT READY
	PUL	B		;CHAR BACK TO B REG
	STA	B, ACIADA	;OUTPUT CHARACTER
	RTS
;***************************************
;* EXAMINE AND DEPOSIT NEXT
;* USES CONTENTS OF XHI & XLO AS POINTER
;***************************************
NCHANG LDX	XHI		;INCREMENT POINTER
	INX
	STX	XHI
	LDA	A, XHI
	BSR	OUT2H		;PRINT OUT ADDRESS
	LDA	A, XLOW
	BSR	OUT2H
	FCB	$8C		;USE CPX SKIP TRICK
;*******************
;* EXAMINE & DEPOSIT
;*******************
CHANGE BSR	BADDR		;BUILD ADDRESS
	BSR	OUTS		;PRINT SPACE
	LDA	A, 0, X	;BYTE INTO A
	BSR	OUT2H		;PRINT BYTE
	BSR	OUTS		;PRINT SPACE
	BSR	BYTE		;GET NEW BYTE
	STA	B, 0, X	;STORE NEW BYTE
;*******************************
;* Command Decoding Section
;*
;* > - Prompt Character
;* L - Load Paper Tape Image
;* J - Jump to specified address
;* M - Examine and Deposit
;* N - Examine and Deposit Next
;* P - Procede from breakpoint
;* List image options
;* Load image
;*******************************
CRLF	LDS	SAVSTK
	LDA	B, #$0D	;CARRIAGE RETURN
	BSR	OUTCH
	LDA	B, #$0A	;LINE FEED
	BSR	OUTCH
	LDX	#Prompt	;Print "FLASH" to make "FLASH>"
	JSR	OUTST
	LDA	B, #'>'	;PROMPT CHARACTER
	BSR	OUTCH
	JSR	INCH		;READ CHARACTER
	TBA			;MAKE A COPY
	BSR	OUTS		;PRINT SPACE
	CMP	A, #'L'
	BEQ	LLOAD		;LOAD PAPER TAPE
	CMP	A, #'?'
	BEQ	PRHLP		;Print Help Message
	;CMP	A, #'R'
	;BEQ	PROT		;Protect Flash
	;CMP	A, #'W'
	;BEQ	UPROT		;Unprotect Flash
	CMP	A, #'J'
	BNE	NOTJ
	BSR	BADDR		;GET ADDRESS TO JUMP TO
	JMP	0, X		;JUMP TO IT
NOTJ	CMP	A, #'M'
	BEQ	CHANGE	;EXAMINE & DEPOSIT
	CMP	A, #'N'
	BEQ	NCHANG	;E & D NEXT
	CMP	A, #'F'	;Flash Read
	BEQ	FRJMP
	CMP	A, #'P'
	BNE	CRLF
	RTI			;PROCEDE FROM BREAKPOINT

FRJMP	JMP	FINIT

PRHLP	LDX	#HelpMsg	;Print Help Message
	JSR	OUTST
	JMP	CRLF

PAGE	MACRO	VAR
	LDA	A, #VAR
	STA	A, $C0FF
	ENDM
STORE	MACRO	LOC, DATA
	LDA	A, #DATA
	STA	A, LOC
	ENDM

KNOCK	MACRO	PAGE, LOC, DATA
	LDA	A, #PAGE
	STA	A, $C0FF
	LDA	A, #DATA
	STA	A, LOC
	ENDM

FCMD	MACRO	COMMAND
	KNOCK	$01, $9555, $AA     ;Write 0xAA to 0x8000 + 0x1555 = 0x9555, page 0x01
	KNOCK $00, $AAAA, $55     ;Write 0x55 to 0x8000 + 0x2AAA = 0xAAAA, page 0x00
	KNOCK $01, $9555, COMMAND ;Write Command to 0x8000 + 0x1555 = 0x9555, page 0x01
	ENDM

PROT	LDX	#Protect	;Print Help Message
	JSR	OUTST
	FCMD	$A0		;Unprotect flash
	PAGE	$02		;Set page to 02
	LDX	#$8000
	LDA	A, #$00
PROTL	STA	A, 0, X
	INX
	INCA
	CPX	#$8100
	BEQ	PROTD
	JMP	PROTL
PROTD	LDA	A, $8000
	NOP
	LDA	B, $8000
	CBA
	BEQ	PROTE
	INC	$3507
	JMP	PROTD
PROTE	LDX	#DoneMsg	;Print Help Message
	JSR	OUTST
	JMP	CRLF

;1000100000000000

UPROT	LDX	#Unprotect	;Print Help Message
	JSR	OUTST
	FCMD	$80
	FCMD	$20
	LDA	A, #$00
	LDX	#$8000
UPRTL	STA	A, 0, X
	INX
	INCA
	CPX	#$8100
	BEQ	UPRTD
	JMP	UPRTL
UPRTD	LDX	#DoneMsg	;Print Help Message
	BSR	OUTST
	JMP	CRLF

;S10400F3FF09

;S10400F30008

;*******************
;* RESET ENTRY POINT
;*******************
;RESET LDS	#ECHO		;INITIALIZE STACK POINTER
;	LDA	B, #$03	;INIT ECHO AND BRKADR FLAGS
;	PSH	B
;	PSH	B
;	STA	B, ACIACS	;MASTER RESET ACIA
;	LDA	B, STRAPS	;LOOK AT STRAPS
;	BMI	JNOTERM	;NO TERM - JUMP TO 0
;	AND	B, #$04	;GET # OF STOP BITS
;	ORA	B, #$D1
;	STA	B, ACIACS	;INIT ACIA PORT
;********************************
;* SOFTWARE INTERRUPT ENTRY POINT
;********************************
;INTRPT STS	SAVSTK	;SAVE STACK POINTER
;	STS	XHI		;SAVE SP FOR N COMMAND
;	LDA	B, BRKADR	;IF BIT 7 OF BRKADR IS SET
;	BMI	JNOTERM	;JUMP TO 0
;	BRA	CRLF		;GOTO COMMAND DECODER
;********************************
;* NOW COME THE INTERRUPT VECTORS
;********************************
;	ORG	$FFF8
;	FDB	MIVEC		;MI VECTOR
;	FDB	INTRPT	;SWI VECTOR
;	FDB	NMIVEC	;NMI VECTOR
;	FDB	RESET		;RESET VECTOR

;	END

OUTST	LDA	B, 0, X
	CMP	B, #0
	BEQ	ENDST
	PSH	B		;SAVE CHAR
OUTS1 JSR	POLCAT	;ACIA STATUS TO B REG
	ASR	B
	BCC	OUTS1		;XMIT NOT READY
	PUL	B		;CHAR BACK TO B REG
	STA	B, ACIADA	;OUTPUT CHARACTER
	INX
	JMP	OUTST
ENDST	RTS

;****************************************************
;* Load Paper Tape Image from Flash Memory
;* Load ONLY S1 Type Records 
;* Terminate on S9, checksum error, or non-hex digits
;****************************************************
FINIT	JSR	BYTE
	STA	B, PAGE
	STA	B, $C0FF
	LDX	#$8100	;Location in flash window where data starts
	STX	FREAD		;Store to Flash read pointer
	
FLOAD	BSR	FINCH		;Read Byte from flash
	SUB	B, #'S'	; Is it 'S'?
	BNE	FLOAD		;  No, first char is not 'S'
	BSR	FINCH		;  Yes, Read next byte from flash
	CMP	B, #'9'	;  Is it '9'?
	BEQ	FC1		;   Yes, End of Record
	CMP	B, #'1'	;  Is it '1'?
	BNE	FLOAD		;   No, go back to Load
	CLR	A		;Zero the checksum (A)
	BSR	FBYTE		;Read byte count
	SUB	B, #$02	; Subtract 2 in advance for getting the address below
	STA	B, BYTECT	; Store the byte count
	BSR	FBYTE		;Get High Order Address for storing the record
	STA	B, XHI	; Store it
	BSR	FBYTE		;Get Low Order Address for storing the record
	STA	B, XLOW	; Store it
	LDX	XHI		;Load X with built address
FLOAD1 BSR	FBYTE		;Get a data byte
	DEC	BYTECT	; Decrement byte count
	BEQ	FLOAD5	;  Branch if EQual (DEC went 0) Done with this block
	STA	B, 0, X	;Store Register B to memory location X
	INX			;Increment memory location pointer X
	BRA	FLOAD1	;Loop for more data
FLOAD5 INC	A		;Increment checksum
FLLOAD BEQ	FLOAD		;Checksum OK - Its zero
FC1	LDX	#Loaded	;Print Message
	BSR	OUTST
	JMP	CRLF		;Checksum Error - Quit
;**********************************************************
;* Read two bytes from Flash (2 HEX digits) into Register B
;* Register A is used for paper tape checksum
;**********************************************************
FBYTE	BSR	FINHEX	;Get first HEX digit
	ASL	B		;Shift digit to upper 4 bits
	ASL	B		;2
	ASL	B		;3
	ASL	B		;4
	ABA			;Add to checksum
	STA	B, TEMP	;Store digit
	BSR	FINHEX	;Get second HEX digit
	ABA			;Add to checksum
	ADD	B, TEMP	;Combine HEX digits to one byte
	RTS			;Return
;***********************************************
;* Read one HEX digit from Flash into Register B
;* Return to calling program if character
;* received is a HEX digit.  If not, go to CRLF
;***********************************************
FINHEX BSR	FINCH		;Get a character from Flash
	SUB	B, #'0'	;Subtract ASCII zero (0x30) from character
	BMI	FC1		; Branch if Minus (Not a Hex number or letter)
	CMP	B, #$09	;Compare with 9
	BLE	FINGD		; Branch if Less than or Equal (character is 0-9)
	CMP	B, #$11	;Compare with 17 (48+17=65, ASCII A)
	BMI	FC1		; Branch if MInus (Not a HEX letter)
	CMP	B, #$16	;Compare with 22 (48+22=70, ASCII F)
	BGT	FC1		; Branch if Greater Than (Not a HEX letter)
	SUB	B, #$07	;It's a letter, convert to BCD
FINGD	RTS			;Return

FINCH	STX	XTMP		;Save X in XTMP
	LDX	FREAD		;Load Flash Read Pointer
	LDA	B, 0, X	;Load Byte into B
	INX			;Increment X
	CPX	#$C000	;Compare X with BFFF (end of flash window) (C000 because of INX)
	BNE	FCONT		;If not equal, don't change pointer or or page register
	PSH	B		;Save B due to page update
	LDX	#$8100	;Beginning of data in flash window
	INC	PAGE		;Increment Page memory register
	LDA	B, PAGE	;Load to accumulator
	STA	B, $C0FF	;Store to page register
	PUL	B		;Restore B
FCONT	STX	FREAD		;Store Flash Read Pointer
	LDX	XTMP		;Restore X
	
;	PSH	B		;SAVE CHAR
;OUTF JSR	POLCAT	;ACIA STATUS TO B REG
;	ASR	B
;	BCC	OUTF		;XMIT NOT READY
;	PUL	B		;CHAR BACK TO B REG
;	STA	B, ACIADA	;OUTPUT CHARACTER
	
	RTS
	
StartMsg	FCB	$0D, $0A, $0D, $0A
		FCB	"**********************************************************************", $0D, $0A
		FCB	"*        Modified Altair 680B PROM Monitor, ACIA Version 1.0         *", $0D, $0A
		FCB	"*               for the Mini680 Memory Expansion Card                *", $0D, $0A
		FCB	"*             http://www.stockly.com - Grant Stockly 2008            *", $0D, $0A
		FCB	"*                  J FFAB to return to PROM Monitor                  *", $0D, $0A
		FCB	"*                                                                    *", $0D, $0A
		FCB	"*          WARNING: Flash Monitor uses RAM 0x7000 to 0x7FFF          *", $0D, $0A 
		FCB	"**********************************************************************", $0D, $0A
		FCB	$0D, $0A
		FCB	"Type ? for help...",$0D,$0A, $00
Prompt	FCB	"FLASH",$00
HelpMsg	FCB	$0D, $0A
		FCB	"**************************************", $0D, $0A
		FCB	"* Advanced Altair 680B Flash Monitor *", $0D, $0A
		FCB	"*******************************************", $0D, $0A
		FCB	"* Commands:                               *", $0D, $0A
		FCB	"* > - *Flash* Monitor Prompt Character    *", $0D, $0A
		FCB	"* L - Load Paper Tape Image               *", $0D, $0A
		FCB	"* J address - Jump to specified address   *", $0D, $0A
		FCB	"* M address - Examine and Deposit         *", $0D, $0A
		FCB	"* N - Examine and Deposit Next            *", $0D, $0A
		FCB	"* P - Procede from breakpoint             *", $0D, $0A
		FCB	"*                                         *", $0D, $0A
		FCB	"* Type 'J FFAB' to return to PROM Monitor *", $0D, $0A
		FCB	"*******************************************", $0D, $0A
		FCB	"* Advanced Flash Memory Functions         *", $0D, $0A
		FCB	"**************************************************************", $0D, $0A
		FCB	"* F ## - Load Paper Tape Image from Flash                    *", $0D, $0A
		FCB	"*   01 = MITS Altair 680 Basic V1.1 R3.2 Paper Tape          *", $0D, $0A
		FCB	"*   03 = MITS Altair 680 Basic V1.1 R3.2 KCACR Cassette Tape *", $0D, $0A
		FCB	"*   05 = MITS Altair 680 Assembler / Editor                  *", $0D, $0A
		FCB	"*                                                            *", $0D, $0A
		FCB	"* NOTE:  Do not enter numbers not listed above!              *", $0D, $0A
		;FCB	"* R[W] - Write Protect(R) / Write Enable (W) Flash Memory    *", $0D, $0A
		;FCB	"* B ## - Write Flash Memory Block (see manual for usage)     *", $0D, $0A
		FCB	"**************************************************************", $0D, $0A
		FCB	$00
Loaded	FCB	$0D, $0A
		FCB	"Memory Image Loaded!", $0D, $0A
		FCB	"Use 'J 0000' to boot either basic (F 01 or F 03)", $0D, $0A
		FCB	"Use 'J 0107' to enter the assembler/editor and clear memory (F 05)", $0D, $0A
		FCB	"Use 'J 010A' to enter the assembler/editor without clearing memory (F 05)", $0D, $0A
		FCB	"See documentation for more information", $0D, $0A
		FCB	$0D, $0A
		FCB	$00
Protect	FCB	$0D, $0A
		FCB	"*****Flash Software Data Protection Enable Algorithm", $0D, $0A
		FCB	"- Load 0xAA to 0x5555 (0x9000 + 0x1555 = 0xA555, page 0x01)", $0D, $0A
		FCB	"- Load 0x55 to 0x2AAA (0x9000 + 0x2AAA = 0xBAAA, page 0x00)", $0D, $0A
		FCB	"- Load 0xA0 to 0x5555", $0D, $0A
		FCB	"- Load 256 bytes of data to any page (must be done)", $0D, $0A
		FCB	$00
Unprotect	FCB	$0D, $0A
		FCB	"*****Flash Software Data Protection Disable Algorithm", $0D, $0A
		FCB	"- Load 0xAA to 0x5555 (0x9000 + 0x1555 = 0xA555, page 0x01)", $0D, $0A
		FCB	"- Load 0x55 to 0x2AAA (0x9000 + 0x2AAA = 0xBAAA, page 0x00)", $0D, $0A
		FCB	"- Load 0x80 to 0x5555", $0D, $0A
		FCB	"- Load 0xAA to 0x5555", $0D, $0A 
		FCB	"- Load 0x55 to 0x2AAA", $0D, $0A
		FCB	"- Load 0x20 to 0x5555", $0D, $0A
		FCB	"- Load 256 bytes of data to any page (must be done)", $0D, $0A
		FCB	$00
DoneMsg	FCB	"- Done!", $0D, $0A, $0D, $0A, $00


;0x8800 - 0xC7FF
;0x0000 - 0x3FFF

;0x4000 - 0x7FFF

	FCB	[(*+1023)&$fffffc00-*]$01	;Fill the rest of the page with NOP
	FCB	[1024]$01	;Fill the rest of the page with NOP
;
